@@ -45,7 +45,6 @@
 #include "htmlrule.h"
 #include "htmltype.h"
 #include "htmlsettings.h"
-#include "htmlutils.h"
 
 #include "gtkhtmldebug.h"
 
@@ -54,6 +53,8 @@ HTMLObjectClass html_object_class;
 
 #define HO_CLASS(x) HTML_OBJECT_CLASS (HTML_OBJECT (x)->klass)
 
+static void remove_child (HTMLObject *self, HTMLObject *child) G_GNUC_NORETURN;
+
 static void
 destroy (HTMLObject *self)
 {
@@ -354,7 +355,7 @@ relayout (HTMLObject *self,
 	  HTMLEngine *engine,
 	  HTMLObject *child)
 {
-	/* FIXME int types of this stuff might change in `htmlobject.h',
+	/* FIXME gint types of this stuff might change in `htmlobject.h',
            remember to sync.  */
 	guint prev_width;
 	guint prev_ascent, prev_descent;
@@ -735,14 +736,14 @@ html_object_real_backspace (HTMLObject *self, HTMLCursor *cursor, HTMLEngine *en
 	return TRUE;
 }
 
-static int
-html_object_real_get_right_edge_offset (HTMLObject *o, HTMLPainter *painter, int offset)
+static gint
+html_object_real_get_right_edge_offset (HTMLObject *o, HTMLPainter *painter, gint offset)
 {
 	return html_object_get_length (o);
 }
 
-static int
-html_object_real_get_left_edge_offset (HTMLObject *o, HTMLPainter *painter, int offset)
+static gint
+html_object_real_get_left_edge_offset (HTMLObject *o, HTMLPainter *painter, gint offset)
 {
 	return 0;
 }
@@ -947,7 +948,6 @@ html_object_set_parent (HTMLObject *o, HTMLObject *parent)
 	o->parent = parent;
 }
 
-
 static void
 frame_offset (HTMLObject *o,
 	      gint *x_return, gint *y_return)
@@ -981,7 +981,7 @@ html_object_calc_abs_position (HTMLObject *o,
 }
 
 void
-html_object_calc_abs_position_in_frame (HTMLObject *o, int *x_return, int *y_return)
+html_object_calc_abs_position_in_frame (HTMLObject *o, gint *x_return, gint *y_return)
 {
 	HTMLObject *p;
 
@@ -1150,7 +1150,7 @@ html_object_calc_preferred_width (HTMLObject *o,
 
 #if 0
 gint
-html_object_get_uris (HTMLObject *o, char **link, char **target, char **src)
+html_object_get_uris (HTMLObject *o, gchar **link, gchar **target, gchar **src)
 {
 	return TRUE;
 }
@@ -1172,16 +1172,11 @@ gchar *
 html_object_get_complete_url (HTMLObject *o, gint offset)
 {
 	const gchar *url, *target;
-	gchar *url2, *url3;
 
 	url = html_object_get_url (o, offset);
 	target = html_object_get_target (o, offset);
-	url2 = url || target ? g_strconcat (url ? url : "#", url ? (target && *target ? "#" : NULL) : target,
-					    url ? target : NULL, NULL) : NULL;
-	url3 = html_utils_maybe_unescape_amp (url2);
-	g_free (url2);
-
-	return url3;
+	return url || target ? g_strconcat (url ? url : "#", url ? (target && *target ? "#" : NULL) : target,
+					      url ? target : NULL, NULL) : NULL;
 }
 
 const gchar *
@@ -1802,9 +1797,9 @@ html_object_set_data_nocp (HTMLObject *object, const gchar *key, const gchar *va
 }
 
 void
-html_object_set_data_full_nocp (HTMLObject *object, const gchar *key, const gpointer value, GDestroyNotify func)
+html_object_set_data_full_nocp (HTMLObject *object, const gchar *key, gconstpointer value, GDestroyNotify func)
 {
-	g_datalist_set_data_full (&object->object_data_nocp, key, value, func);
+	g_datalist_set_data_full (&object->object_data_nocp, key, (gpointer) value, func);
 }
 
 gpointer
@@ -1820,9 +1815,9 @@ html_object_set_data (HTMLObject *object, const gchar *key, const gchar *value)
 }
 
 void
-html_object_set_data_full (HTMLObject *object, const gchar *key, const gpointer value, GDestroyNotify func)
+html_object_set_data_full (HTMLObject *object, const gchar *key, gconstpointer value, GDestroyNotify func)
 {
-	g_datalist_set_data_full (&object->object_data, key, value, func);
+	g_datalist_set_data_full (&object->object_data, key, (gpointer) value, func);
 }
 
 gpointer
@@ -1857,8 +1852,11 @@ object_save_data (GQuark key_id, gpointer data, gpointer user_data)
 	/* printf ("object %s %s\n", g_quark_to_string (key_id), str); */
 	if (!str) {
 		/* printf ("save %s %s -> %s\n", state->save_data_class_name, g_quark_to_string (key_id), (gchar *) data); */
-		html_engine_save_output_string (state, "<!--+GtkHTML:<DATA class=\"%s\" key=\"%s\" value=\"%s\">-->",
-						state->save_data_class_name, g_quark_to_string (key_id), (char *) data);
+		html_engine_save_delims_and_vals (state,
+				"<!--+GtkHTML:<DATA class=\"", state->save_data_class_name,
+				"\" key=\"", g_quark_to_string (key_id),
+				"\" value=\"", (gchar *) data,
+				"\">-->", NULL);
 		html_engine_set_class_data (state->engine, state->save_data_class_name, g_quark_to_string (key_id), data);
 	}
 }
@@ -1873,13 +1871,18 @@ handle_object_data (gpointer key, gpointer value, gpointer data)
 	/* printf ("handle: %s %s %s %s\n", state->save_data_class_name, key, value, str); */
 	if (!str) {
 		/* printf ("clear\n"); */
-		html_engine_save_output_string (state, "<!--+GtkHTML:<DATA class=\"%s\" clear=\"%s\">-->",
-						state->save_data_class_name, (char *) key);
+		html_engine_save_delims_and_vals (state,
+				"<!--+GtkHTML:<DATA class=\"", state->save_data_class_name,
+				"\" clear=\"", (gchar *) key,
+				"\">-->", NULL);
 		state->data_to_remove = g_slist_prepend (state->data_to_remove, key);
 	} else if (strcmp (value, str)) {
 		/* printf ("change\n"); */
-		html_engine_save_output_string (state, "<!--+GtkHTML:<DATA class=\"%s\" key=\"%s\" value=\"%s\">-->",
-						state->save_data_class_name, (char *) key, str);
+		html_engine_save_delims_and_vals (state,
+				"<!--+GtkHTML:<DATA class=\"", state->save_data_class_name,
+				"\" key=\"", (gchar *) key,
+				"\" value=\"", str,
+				"\">-->", NULL);
 		html_engine_set_class_data (state->engine, state->save_data_class_name, key, value);
 	}
 }
@@ -1949,7 +1952,6 @@ html_object_check_cut_lists (HTMLObject *self, HTMLObject *replacement, GList *l
 		right->data = replacement;
 }
 
-
 typedef struct {
 	HTMLInterval *i;
 	GString *buffer;
@@ -2127,7 +2129,7 @@ html_object_get_insert_level (HTMLObject *o)
 	switch (HTML_OBJECT_TYPE (o)) {
 	case HTML_TYPE_TABLECELL:
 	case HTML_TYPE_CLUEV: {
-		int level = 3;
+		gint level = 3;
 
 		while (o && (HTML_IS_CLUEV (o) || HTML_IS_TABLE_CELL (o))
 		       && HTML_CLUE (o)->head && (HTML_IS_CLUEV (HTML_CLUE (o)->head) || HTML_IS_TABLE_CELL (HTML_CLUE (o)->head))) {
@@ -2209,7 +2211,6 @@ html_object_get_child (HTMLObject *self, gint index)
 	return HO_CLASS (self)->get_child ? (* HO_CLASS (self)->get_child) (self, index) : NULL;
 }
 
-
 gint
 html_object_get_child_index (HTMLObject *self, HTMLObject *child)
 {
@@ -2278,14 +2279,14 @@ html_object_get_direction (HTMLObject *o)
 	return (* HO_CLASS (o)->get_direction) (o);
 }
 
-const char *
+const gchar *
 html_object_get_id (HTMLObject *o)
 {
 	return o->id;
 }
 
 void
-html_object_set_id (HTMLObject *o, const char *id)
+html_object_set_id (HTMLObject *o, const gchar *id)
 {
 	g_free (o->id);
 	o->id = g_strdup (id);
@@ -2300,14 +2301,14 @@ html_object_get_flow (HTMLObject *o)
 	return HTML_CLUEFLOW (o);
 }
 
-int
-html_object_get_right_edge_offset (HTMLObject *o, HTMLPainter *painter, int offset)
+gint
+html_object_get_right_edge_offset (HTMLObject *o, HTMLPainter *painter, gint offset)
 {
 	return (* HO_CLASS (o)->get_right_edge_offset) (o, painter, offset);
 }
 
-int
-html_object_get_left_edge_offset (HTMLObject *o, HTMLPainter *painter, int offset)
+gint
+html_object_get_left_edge_offset (HTMLObject *o, HTMLPainter *painter, gint offset)
 {
 	return (* HO_CLASS (o)->get_left_edge_offset) (o, painter, offset);
 }
